//
//  ViewController.m
//  TestBeeWi
//
//  Created by Pierre Emmanuel Aguera on 30/05/2017.
//  Copyright © 2017 Pierre Emmanuel Aguera. All rights reserved.
//

#import "ViewController.h"


#define BEEWI_SERVICE_1_String  @"A8B3FA04-4834-4051-89D0-3DE95CDDD318"
#define BEEWI_SystemID_String  @"180A"

// Reference for data reading : https://github.com/enrimilan/BeeWi-BBW200-Reader
#define BEEWI_DATA_Characteristic @"A8B3FB43-4834-4051-89D0-3DE95CDDD318"


#define NO_PERIPH @"Pas de périphérique"

typedef enum {UndefinedState=0, InitializedState, SearchingState, PeriphFoundState, ConnectedState, GettingDescState, DescFoundState, ReadingState} appState_t;



@interface ViewController ()

@property IBOutlet UIButton *searchBtn;
@property IBOutlet UIButton *connectBtn;
@property IBOutlet UIButton *descBtn;
@property IBOutlet UIButton *readBtn;
@property IBOutlet UIButton *disconBtn;

@property IBOutlet UILabel *tempLab;
@property IBOutlet UILabel *humidLab;
@property IBOutlet UILabel *batLab;
@property IBOutlet UILabel *periphLab;

@property CBCentralManager *centralManager;
@property CBPeripheral *thermPeripheral;
@property NSArray *services;
@property CBCharacteristic *dataChar;
@property float temperature;
@property int humidity;
@property int battery;
@property NSString *periphName;

@property appState_t appState;

@end

@implementation ViewController



- (id)initWithCoder:(NSCoder *)aDecoder {
	self = [super initWithCoder:aDecoder];

	_services = @[[CBUUID UUIDWithString:BEEWI_SERVICE_1_String]];
	
	_centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
	
	// Peripheral initialization
	_thermPeripheral = nil;
	
	// Data characteristic initialization
	_dataChar = nil;
	
	_appState = UndefinedState;
	
	_periphName = NO_PERIPH;
	
	_temperature = 0.0;
	_humidity = 0;
	_battery = 0;

	return self;
}


- (void)viewDidLoad {
	[super viewDidLoad];
	// Do any additional setup after loading the view, typically from a nib.
	
	// Show value in labels
	[self updateInterface];
}


- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
	// Dispose of any resources that can be recreated.
}


- (void)updateInterface {
	if ((_thermPeripheral != nil) && (_dataChar != nil)) {
		[_tempLab setText:[NSString stringWithFormat:@"Température %.1f °C", _temperature]];
		[_humidLab setText:[NSString stringWithFormat:@"Humidité %d %%", _humidity]];
		[_batLab setText:[NSString stringWithFormat:@"Batterie %d %%", _battery]];
	} else {
		[_tempLab setText:@"Température --.-"];
		[_humidLab setText:@"Humidité --"];
		[_batLab setText:@"Batterie --"];
	}
	switch (_appState) {
		case UndefinedState:
		case InitializedState:
			[_searchBtn setEnabled:YES];
			[_connectBtn setEnabled:NO];
			[_descBtn setEnabled:NO];
			[_readBtn setEnabled:NO];
			[_disconBtn setEnabled:NO];
			
			_periphName = @"Pas de périphérique";
			[_periphLab setText:_periphName];
			[_periphLab setTextColor:[UIColor darkGrayColor]];
			break;
		case SearchingState:
			[_searchBtn setEnabled:YES];
			[_connectBtn setEnabled:NO];
			[_descBtn setEnabled:NO];
			[_readBtn setEnabled:NO];
			[_disconBtn setEnabled:NO];
			
			[_periphLab setText:_periphName];
			[_periphLab setTextColor:[UIColor darkGrayColor]];
			break;
		case PeriphFoundState:
			[_searchBtn setEnabled:YES];
			[_connectBtn setEnabled:YES];
			[_descBtn setEnabled:NO];
			[_readBtn setEnabled:NO];
			[_disconBtn setEnabled:NO];
			
			[_periphLab setText:_periphName];
			[_periphLab setTextColor:[UIColor redColor]];
			break;
		case ConnectedState:
			[_searchBtn setEnabled:NO];
			[_connectBtn setEnabled:NO];
			[_descBtn setEnabled:YES];
			[_readBtn setEnabled:NO];
			[_disconBtn setEnabled:YES];

			[_periphLab setText:_periphName];
			[_periphLab setTextColor:[UIColor greenColor]];
			break;
		case GettingDescState:
			[_searchBtn setEnabled:NO];
			[_connectBtn setEnabled:NO];
			[_descBtn setEnabled:NO];
			[_readBtn setEnabled:NO];
			[_disconBtn setEnabled:YES];
			
			[_periphLab setText:_periphName];
			[_periphLab setTextColor:[UIColor greenColor]];
			break;
		case DescFoundState:
			[_searchBtn setEnabled:NO];
			[_connectBtn setEnabled:NO];
			[_descBtn setEnabled:NO];
			[_readBtn setEnabled:YES];
			[_disconBtn setEnabled:YES];
			
			[_periphLab setText:_periphName];
			[_periphLab setTextColor:[UIColor greenColor]];
			break;
		case ReadingState:
			[_searchBtn setEnabled:NO];
			[_connectBtn setEnabled:NO];
			[_readBtn setEnabled:YES];
			[_disconBtn setEnabled:YES];
			
			[_periphLab setText:_periphName];
			[_periphLab setTextColor:[UIColor greenColor]];
			break;
	}
}


#pragma mark - CBCentralManagerDelegate

// method called whenever you have successfully connected to the BLE peripheral
- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral {
	NSLog(@"Central manager did connect to peripheral");
	_appState = ConnectedState;
	
	// Update interface
	[self updateInterface];
}


- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
	NSLog(@"Central manager failed to connect to peripheral");
	_appState = InitializedState;
	
	// Update interface
	[self updateInterface];
}


- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
	NSLog(@"Central manager did disconnect from peripheral");
	_appState = PeriphFoundState;
	
	// Update interface
	[self updateInterface];
}


// CBCentralManagerDelegate - This is called with the CBPeripheral class as its main input parameter. This contains most of the information there is to know about a BLE peripheral.
- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI {
	NSString *localName = [advertisementData objectForKey:CBAdvertisementDataLocalNameKey];
	if ([localName length] > 0) {
		NSLog(@"Found the device: %@", localName);
		if ([localName compare:@"BeeWi SmartClim"] == NSOrderedSame) {
			_periphName = localName;
			
			// End scanning
			[_centralManager stopScan];

			// Retain peripheral
			_thermPeripheral = peripheral;
			
			_appState = PeriphFoundState;

			// Update interface
			[self updateInterface];
		}
	}
}


// method called whenever the device state changes.
- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
	// Determine the state of the central
	switch ([central state]) {
		case CBManagerStatePoweredOn:
			NSLog(@"CoreBluetooth BLE hardware is powered on and ready");
			_appState = InitializedState;
			break;
		case  CBManagerStatePoweredOff:
			NSLog(@"CoreBluetooth BLE hardware is powered off");
			_appState = UndefinedState;
			break;
		case CBManagerStateUnauthorized:
			NSLog(@"CoreBluetooth BLE state is unauthorized");
			_appState = UndefinedState;
			break;
		case CBManagerStateUnknown:
			NSLog(@"CoreBluetooth BLE state is unknown");
			_appState = UndefinedState;
			break;
		case CBManagerStateUnsupported:
			NSLog(@"CoreBluetooth BLE hardware is unsupported on this platform");
			_appState = UndefinedState;
			break;
		case CBManagerStateResetting:
			NSLog(@"CoreBluetooth BLE hardware is resetting");
			_appState = UndefinedState;
			break;
	}
	// Update interface
	[self updateInterface];
}


#pragma mark - CBPeripheralDelegate

// CBPeripheralDelegate - Invoked when you discover the peripheral's available services.
- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error {
	for (CBService *service in peripheral.services) {
		NSLog(@"Discovered service %@", service);
		NSLog(@"Discovering characteristics for service %@", service);
		[peripheral discoverCharacteristics:nil forService:service];
	}
}


// Invoked when you discover the characteristics of a specified service.
- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error {
	for (CBCharacteristic *characteristic in service.characteristics) {
		NSLog(@"Discovered characteristic %@", characteristic);
		if ([[[characteristic UUID] UUIDString] compare:BEEWI_DATA_Characteristic] == NSOrderedSame) {
			_dataChar = characteristic;
			
			_appState = DescFoundState;

			// Update GUI
			[self updateInterface];
		}
  }
}


// Invoked when you retrieve a specified characteristic's value, or when the peripheral device notifies your app that the characteristic's value has changed.
- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
	if ([characteristic UUID] == [_dataChar UUID]) {
		NSData *data = characteristic.value;
		if ([data length] != 10) {
			NSLog(@"bad data length %d (should be 20)", (int)[data length]);
		} else {
/*# the temperature consists of 3 bytes
# Posivive value: byte 1 & 2 present the tenfold of the temperature
# Negative value: byte 2 - byte 3 present the tenfold of the temperature */
			int temperature_buf[3];
			char *ptr_data_buf = (char *)[data bytes];
			temperature_buf[0] = ((int)ptr_data_buf[0]) & 0x000000ff;
			temperature_buf[1] = ((int)ptr_data_buf[1]) & 0x000000ff;
			temperature_buf[2] = ((int)ptr_data_buf[2]) & 0x000000ff;
			int temperatureInt = (temperature_buf[2]*255)+temperature_buf[1];
			_temperature = (float)temperatureInt / 10.0;
			
			char humidity_buf[1];
			humidity_buf[0] = ptr_data_buf[4];
			_humidity = (char)*humidity_buf;
			
			char battery_buf[1];
			battery_buf[0] = ptr_data_buf[9];
			_battery = (char)*battery_buf;
			
			_appState = DescFoundState;
			
			// Update GUI (labels)
			[self updateInterface];
		}
	}
}


#pragma mark - IBActions
- (IBAction)searchPeripheral:(id)sender {
	// Doesn't advertise for service
	[_centralManager scanForPeripheralsWithServices:nil options:nil];
	
	_appState = SearchingState;
	// Update interface
	[self updateInterface];
}


- (IBAction)connectToPeripheral:(id)sender {
	// Connect
	_thermPeripheral.delegate = self;
	
	[_centralManager connectPeripheral:_thermPeripheral options:nil];
}

- (IBAction)readDescFromPeripheral:(id)sender {
	// Read service and characteristic description
	[_thermPeripheral discoverServices:_services];
	
	_appState = GettingDescState;
	// Update interface
	[self updateInterface];
}


- (IBAction)readData:(id)sender {
	if ((_thermPeripheral != nil) && (_dataChar != nil)) {
		_appState = ReadingState;
		// Update interface
		[self updateInterface];
		
		[_thermPeripheral readValueForCharacteristic:_dataChar];
	}
}


- (IBAction)disconnectFromPeripheral:(id)sender {
	// Disconnect
	[_centralManager cancelPeripheralConnection:_thermPeripheral];
	_dataChar = nil;
}

@end
